% This is the main run file for estimating the trivariate unobserved 
% components model of inflation, GDP and unemployment in Grant and Chan (2017).
%
% This code is free to use for academic purposes only, provided that the 
% paper is cited as:
%
% Grant, A.L. and Chan, J.C.C. (2017). Measuring the Output Gap Using 
% Stochastic Model Specification Search, CAMA Working Paper 2/2017.
%
% This code comes without technical support of any kind.  It is expected to
% reproduce the results reported in the paper. Under no circumstances will
% the authors be held responsible for any use (or misuse) of this code in
% any way.

clear; clc;
nsims = 50000;
burnin = 1000;

USdata = xlsread('USdata.xls', 'B2:E277');
y = 100*log(USdata(5:end,1));
pi = 400*log(USdata(5:end,2)./USdata(4:end-1,2));
u = USdata(5:end,3);
x = USdata(5:end,4);
T = size(y,1);
T0 = 127; %1979Q3
break_dates = [93; 237];  % 1971Q1, 2007Q1
D = [break_dates(1) <= (1:T)' break_dates(2)<=(1:T)'];
tid = linspace(1948,2015.75,T)';
    % prior
pistar0 = 3; Vpistar = 100;
ustar0 = 3; Vustar = 100;
ystar00 = 758; Vystar0 = 100;
rhoc0 = zeros(2,1); iVrhoc = speye(2);
rhou0 = zeros(2,1); iVrhou = speye(2);
gamu0 = 0; Vgamu = 1;
rhopi0 = 0; Vrhopi = 1;
lamu0 = 0; Vlamu = 1;
lamy0 = 0; Vlamy = 1;
a00 = 0; Va0 = 1;
d0 = zeros(2,1); Vd = ones(2,1);
b0 = [0 1]'; Vb = 100*ones(2,1);
nu_sigx2 = 3; S_sigx2 = .2;
Vsigpistar = .2^2;
Vsigystar = .1^2;
Vsigustar = .1^2;
Vsiga = .01^2;
Vsighpi = .1^2;
Vsighc = .1^2;
Vsighu = .1^2;
Vhpi = 100;
Vhc = 100;
Vhu = 100;
p = .5*ones(11,1);
nu0_c = 1/8;
nu0_pi = 1/8;
nu0_u = 1/8;

disp('Starting MCMC for UC-SMSS.... ');
start_time = clock; 
 
    % initialize the Markov chain
rhopi = .7;
rhou = [1.2 -.3]';
rhoc = [1.35 -.7]';
lamu = .5;
gamu = -2;
lamy = .4;
b = [0 1]';
sigx2 = var(x(T0+1:end));
sighc2 = .1;
sighpi2 = .1;
sighu2 = .1;
sigystar = .5;
sigpistar2 = .05;
sigustar2 = .01;
siga = .1;
hpi = log(var(pi))*ones(T,1);
hc = log(var(y))*ones(T,1);
hu = log(var(u))*ones(T,1);
ustar = 6*ones(T,1);
pistar = 5*ones(T,1);
ystar = y;
ystar0 = y(1);
a0 = .6;
d = [-.1 -.1]';
nuc = 5;
nupi = 5;
nuu = 40;
zc = 1./gamrnd(nuc/2,2/nuc,T,1);
zpi = 1./gamrnd(nupi/2,2/nupi,T,1);
zu = 1./gamrnd(nuu/2,2/nuu,T,1);
del = ones(11,1);

    % construct a few things before MCMC
S = [sparse(T-T0,T0) speye(T-T0)];
SS = S'*S;
H = speye(T) - sparse(2:T,1:T-1,ones(1,T-1),T,T);
H2 = H^2;
HH = H'*H;
Dtilde = D.*repmat((1:T)',1,2);

Hrhopi = speye(T) - rhopi*sparse(2:T,1:T-1,ones(1,T-1),T,T);
Hrhou = speye(T) - rhou(1)*sparse(2:T,1:T-1,ones(1,T-1),T,T) ...
    - rhou(2)*sparse(3:T,1:T-2,ones(1,T-2),T,T);
Hrhoc = speye(T) - rhoc(1)*sparse(2:T,1:T-1,ones(1,T-1),T,T) ...
    - rhoc(2)*sparse(3:T,1:T-2,ones(1,T-2),T,T);

atilde = chol(HH,'lower')'\randn(T,1);
iSpi = sparse(1:T,1:T,1./(exp(hpi).*zpi));
iSc = sparse(1:T,1:T,1./(exp(hc).*zc));
iSu = sparse(1:T,1:T,1./(exp(hu).*zu));


    % initialize for storeage
store_theta = zeros(nsims,25); % [rhopi; rhoc; rhou; lamu; gamu; lamy; sighpi2; sighc2; sighu2; sigpistar2;
                               % sigystar; sigustar2; sighu2; siga; a0; d; nuc; nupi; nuu; b; sigx2];
store_pistar = zeros(nsims,T); 
store_ystar = zeros(nsims,T); 
store_mu = zeros(nsims,T); 
store_ustar = zeros(nsims,T); 
store_hc = zeros(nsims,T); 
store_hpi = zeros(nsims,T); 
store_hu = zeros(nsims,T); 
store_del = zeros(nsims,11); 
store_a = zeros(nsims,T);
countrhoc = 0;
countrhou = 0;
countsigustar2 = 0;
countsigpistar2 = 0;
countsigystar2 = 0;
countsighpi2 = 0;
countsighc2 = 0;
countsighu2 = 0;
countnuc = 0;
countnupi = 0;
countnuu = 0;

rand('state', sum(100*clock) ); randn('state', sum(200*clock) );
    
for isim = 1:nsims+burnin   
    
        % sample pistar    
    mu_pis = H\[pistar0;sparse(T-1,1)];
    m_pis = Hrhopi\(del(1)*lamu*(u-ustar)+del(3)*lamy*(y-ystar));
    HiSpisH = H'*sparse(1:T,1:T,[1/Vpistar 1/sigpistar2*ones(1,T-1)])*H;
    Hrhopi_iSpi = Hrhopi'*iSpi;     
    Kpis = HiSpisH + Hrhopi_iSpi*Hrhopi + b(2)^2/sigx2*SS;
    pishat = Kpis\(HiSpisH*mu_pis + Hrhopi_iSpi*Hrhopi*(pi-m_pis) ...
        + b(2)/sigx2*S'*((x(T0+1:T)-b(1))));
    pistar = pishat + chol(Kpis,'lower')'\randn(T,1);

        % sample ustar
    mu_us = H\[ustar0;sparse(T-1,1)];    
    HiSusH = H'*sparse(1:T,1:T,[1/Vustar 1/sigustar2*ones(1,T-1)])*H;
    HrhouiSuHrhou = Hrhou'*iSu*Hrhou;
    HrhociScHrhoc = Hrhoc'*iSc*Hrhoc;
    Kus = HiSusH + HrhouiSuHrhou + (del(2)*gamu)^2*HrhociScHrhoc + ...
        (del(1)*lamu)^2*iSpi;
    CKus = chol(Kus,'lower');
    ushat = CKus'\(CKus\(HiSusH*mu_us + HrhouiSuHrhou*u ...
        - del(2)*gamu*HrhociScHrhoc*(y-ystar-del(2)*gamu*u) ...
        - del(1)*lamu*iSpi*(Hrhopi*(pi-pistar)-del(3)*lamy*(y-ystar)-del(1)*lamu*u)));
    ustar = ushat + CKus'\randn(T,1);
    
        % sample ystartilde
    m_yst = ystar0 + del(4)*(1:T)'*a0 + del(6)*siga*cumsum(atilde) ...
        + del(7)*d(1)*Dtilde(:,1) + del(8)*d(2)*Dtilde(:,2);
    HrhociScHrhoc = Hrhoc'*iSc*Hrhoc;    
    Kyst = HH + (del(3)*lamy*del(5)*sigystar)^2*iSpi ...
        + (del(5)*sigystar)^2*HrhociScHrhoc;    
    CKyst = chol(Kyst,'lower');    
    ysthat = CKyst'\(CKyst\(-del(3)*lamy*del(5)*sigystar*iSpi*...
        (Hrhopi*(pi-pistar) - del(1)*lamu*(u-ustar) - del(3)*lamy*(y-m_yst)) ...
        + del(5)*sigystar*HrhociScHrhoc*(y - m_yst - del(2)*gamu*(u-ustar))));
    ystartilde = ysthat + CKyst'\randn(T,1);  
    
        % sample beta_ystar (ystar0, a0, sigystar, siga, d)
    [ystar0,a0,sigystar,siga,d] = sample_betays(y,pi,u,gamu,lamy,lamu,...
        ystartilde,pistar,atilde,ustar,Dtilde,Hrhoc,Hrhopi,iSc,iSpi,del,...
        [ystar00 a00 d0' Vystar0 Va0 Vsigystar Vsiga Vd']);
            % permutate ystartilde and sigystar
    U = -1 + 2*(rand>0.5);
    ystartilde = U*ystartilde;
    sigystar = U*sigystar;
    
        % sample atilde    
    HiSH_rhoc = Hrhoc'*iSc*Hrhoc;    
    m_Atilde = ystar0 + del(4)*(1:T)'*a0 + del(5)*sigystar*ystartilde ...
        + del(7)*d(1)*Dtilde(:,1) + del(8)*d(2)*Dtilde(:,2);    
    KAtilde = H2'*H2 + (del(3)*lamy*del(6)*siga)^2*iSpi + (del(6)*siga)^2*HiSH_rhoc;
    Atildehat = KAtilde\(-del(3)*lamy*del(6)*siga*iSpi*...
        (Hrhopi*(pi-pistar)-del(1)*lamu*(u-ustar)-del(3)*lamy*(y-m_Atilde))...
        + del(6)*siga*HiSH_rhoc*(y-m_Atilde-del(2)*gamu*(u-ustar)));
    Atilde = Atildehat + chol(KAtilde,'lower')'\randn(T,1);
    atilde = H*Atilde;    
    
        % sample beta_ystar (ystar0, a0, sigystar, siga, d)
    [ystar0,a0,sigystar,siga,d] = sample_betays(y,pi,u,gamu,lamy,lamu,...
        ystartilde,pistar,atilde,ustar,Dtilde,Hrhoc,Hrhopi,iSc,iSpi,del,...
        [ystar00 a00 d0' Vystar0 Va0 Vsigystar Vsiga Vd']);
       % permutate atilde and siga
    U = -1 + 2*(rand>0.5);
    atilde = U*atilde;
    siga = U*siga;  
        % update ystar
    ystar = ystar0 + del(4)*(1:T)'*a0 + del(6)*siga*cumsum(atilde)...
        + del(5)*sigystar*ystartilde + del(7)*d(1)*Dtilde(:,1) + del(8)*d(2)*Dtilde(:,2);
       
        % sample del_lamu, del_gamu, del_lamy marginal of lamu, gamu, lamy, rhopi
    sample_del_var;    

       % sample rhopi, lamu, lamy, gamu
    Xpi = [[0;pi(1:end-1)-pistar(1:end-1)] del(1)*(u-ustar) del(3)*(y-ystar)];
    K_betapi = diag([1/Vrhopi 1/Vlamu 1/Vlamy]) + Xpi'*iSpi*Xpi;
    betapi_hat = K_betapi\([rhopi0/Vrhopi; lamu0/Vlamu; lamy0/Vlamy] ...
        + Xpi'*iSpi*(pi-pistar));
    betapi = betapi_hat + chol(K_betapi,'lower')'\randn(3,1);
            % update the parameters
    rhopi = betapi(1);
    lamu = betapi(2)*del(1);
    lamy = betapi(3)*del(3);
    Hrhopi = speye(T) - rhopi*sparse(2:T,1:T-1,ones(1,T-1),T,T);   
    
    HrhociScHrhoc = Hrhoc'*iSc*Hrhoc;
    Kgamu = 1/Vgamu + del(2)^2*(u-ustar)'*HrhociScHrhoc*(u-ustar);
    gamuhat = Kgamu\(gamu0/Vgamu + del(2)*(u-ustar)'*HrhociScHrhoc*(y-ystar));
    gamu = (gamuhat + 1/sqrt(Kgamu)*randn)*del(2);    
    
        % sample del_a0, del_ys, del_a marginal of ystar0, a0, sigystar, siga
    sample_del_trend;
    
        % sample beta_ystar (ystar0, a0, sigystar, siga, d)   
    [ystar0,a0,sigystar,siga,d] = sample_betays(y,pi,u,gamu,lamy,lamu,...
        ystartilde,pistar,atilde,ustar,Dtilde,Hrhoc,Hrhopi,iSc,iSpi,del,...
        [ystar00 a00 d0' Vystar0 Va0 Vsigystar Vsiga Vd']);
    
       % sample del_zpi and del_zc magrinally of zpi and zc
    sample_del_dist;  
    
       % sample zpi    
    if del(9) == 1
        zpi = 1./gamrnd((nupi+1)/2,2./(s2_pi./exp(hpi)+nupi));
        iSpi = sparse(1:T,1:T,1./(exp(hpi).*zpi));
    else
        zpi = 1./gamrnd(nupi/2,2/nupi,T,1);
        iSpi = sparse(1:T,1:T,1./exp(hpi));
    end
    
        % sample zc      
    if del(10) == 1
        zc = 1./gamrnd((nuc+1)/2,2./(s2_c./exp(hc)+nuc));
        iSc = sparse(1:T,1:T,1./(exp(hc).*zc));
    else
        zc = 1./gamrnd(nuc/2,2/nuc,T,1);
        iSc = sparse(1:T,1:T,1./exp(hc));
    end 
    
        % sample zu    
    if del(11) == 1
        zu = 1./gamrnd((nuu+1)/2,2./(s2_u./exp(hu)+nuu));
        iSu = sparse(1:T,1:T,1./(exp(hu).*zu));
    else
        zu = 1./gamrnd(nuu/2,2/nuu,T,1);
        iSu = sparse(1:T,1:T,1./exp(hu));
    end 
    
        % sample rhoc
    c = y-ystar-del(2)*gamu*(u-ustar);
    Xrhoc = [[0;c(1:end-1)] [0;0;c(1:end-2)]];
    Krhoc = iVrhoc + Xrhoc'*iSc*Xrhoc;
    rhochat = Krhoc\(iVrhoc*rhoc0 + Xrhoc'*iSc*c);
    flag = 0; count = 0;
    while flag == 0 && count < 100        
        rhocc = rhochat + chol(Krhoc,'lower')'\randn(2,1);
            % check if the draw is stationary
        if sum(rhocc) < .99 && rhocc(2) - rhocc(1) < .99 && rhocc(2) > -.99
            rhoc = rhocc;
            flag = 1;
            countrhoc = countrhoc + 1;
        end
        count = count + 1;
    end
    Hrhoc = speye(T) - rhoc(1)*sparse(2:T,1:T-1,ones(1,T-1),T,T) ...
        - rhoc(2)*sparse(3:T,1:T-2,ones(1,T-2),T,T);
    
       % sample rhou
    Xrhou = [[0;u(1:end-1)-ustar(1:end-1)] [0;0;u(1:end-2)-ustar(1:end-2)]];
    Krhou = iVrhou + Xrhou'*iSu*Xrhou;
    rhouhat = Krhou\(iVrhou*rhou0 + Xrhou'*iSu*(u-ustar));
    flag = 0; count = 0;
    while flag == 0 && count < 100
        rhouc = rhouhat + chol(Krhou,'lower')'\randn(2,1);
            % check if the draw is stationary
        if sum(rhouc) < .99 && rhouc(2) - rhouc(1) < .99 && rhouc(2) > -.99 
            rhou = rhouc;
            flag = 1;
            countrhou = countrhou + 1;
        end
        count = count + 1;
    end
    Hrhou = speye(T) - rhou(1)*sparse(2:T,1:T-1,ones(1,T-1),T,T) ...
        - rhou(2)*sparse(3:T,1:T-2,ones(1,T-2),T,T);    
  
        % sample hc
    s2_c = (Hrhoc*(y-ystar-del(2)*gamu*(u-ustar))).^2;
    wc = log(s2_c./zc);
    hc = SVRW(wc,hc,sighc2,0,Vhc);
    
        % sample hpi
    s2_pi = (Hrhopi*(pi-pistar)-del(1)*lamu*(u-ustar)-del(3)*lamy*(y-ystar)).^2;
    wpi = log(s2_pi./zpi);
    hpi = SVRW(wpi,hpi,sighpi2,0,Vhpi);
    
        % sample hu
    s2_u = (Hrhou*(u-ustar)).^2;
    wu = log(s2_u./zu);
    hu = SVRW(wu,hu,sighu2,0,Vhu);
    
        % sample sigustar2    
    sigustar2c = 1/gamrnd((T-2)/2, 2/sum((ustar(2:end)-ustar(1:end-1)).^2));
    alpMH = exp(-(sigustar2c-sigustar2)/(2*Vsigustar));
    if alpMH > rand
        sigustar2 = sigustar2c;
        countsigustar2 = countsigustar2 + 1;
    end    
    
        % sample sigpistar2    
    sigpistar2c = 1/gamrnd((T-2)/2, 2/sum((pistar(2:end)-pistar(1:end-1)).^2));
    alpMH = exp(-(sigpistar2c-sigpistar2)/(2*Vsigpistar));
    if alpMH > rand
        sigpistar2 = sigpistar2c;
        countsigpistar2 = countsigpistar2 + 1;
    end    
    
        % sample sighpi2    
    sighpi2c = 1/gamrnd((T-2)/2, 2/sum((hpi(2:end)-hpi(1:end-1)).^2));
    alpMH = exp(-(sighpi2c-sighpi2)/(2*Vsighpi));
    if alpMH > rand
        sighpi2 = sighpi2c;
        countsighpi2 = countsighpi2 + 1;
    end    
    
        % sample sighc2    
    sighc2c = 1/gamrnd((T-2)/2, 2/sum((hc(2:end)-hc(1:end-1)).^2));
    alpMH = exp(-(sighc2c-sighc2)/(2*Vsighc));
    if alpMH > rand
        sighc2 = sighc2c;
        countsighc2 = countsighc2 + 1;
    end    
    
        % sample sighu2
    sighu2c = 1/gamrnd((T-2)/2, 2/sum((hu(2:end)-hu(1:end-1)).^2));
    alpMH = exp(-(sighu2c-sighu2)/(2*Vsighu));
    if alpMH > rand
        sighu2 = sighu2c;
        countsighu2 = countsighu2 + 1;
    end

        % sample nupi
    [nupi,flag] = sample_nu(zpi,nu0_pi,nupi);
    countnupi = countnupi + flag;
     
        % sample nuc
    [nuc,flag] = sample_nu(zc,nu0_c,nuc);
    countnuc = countnuc + flag;
    
        % sample nuu
    [nuu,flag] = sample_nu(zu,nu0_u,nuu);
    countnuu = countnuu + flag;
    
        % sample b
    Xb = [ones(T-T0,1), pistar(T0+1:end)];
    Kb = diag(1./Vb) + Xb'*Xb/sigx2;
    bhat = Kb\(b0./Vb + Xb'*x(T0+1:end)/sigx2);
    b = bhat + chol(Kb,'lower')'\randn(2,1);
    
        % sample sigx2
    sigx2 = 1./gamrnd(nu_sigx2+(T-T0)/2, 1/(S_sigx2 + sum((x(T0+1:end)-Xb*b).^2)/2));
    
    if ( mod( isim, 1000 ) ==0 )
         disp(  [ num2str( isim ) ' loops... ' ] )
    end     

    if isim > burnin
        isave = isim - burnin; 
        store_mu(isave,:) = 4*(ystar-[ystar0;ystar(1:end-1)])';
        store_a(isave,:) = 4*(a0*del(4) + del(6)*siga*atilde)';
        store_ystar(isave,:) = ystar';
        store_ustar(isave,:) = ustar';
        store_pistar(isave,:) = pistar'; 
        store_hpi(isave,:) = hpi';         
        store_hc(isave,:) = hc';         
        store_hu(isave,:) = hu'; 
        store_theta(isave,:) = [rhopi; rhoc; rhou; lamu; gamu; lamy; sighpi2; ...
            sighc2; sighu2; sigpistar2; sigystar; sigustar2; sighu2; siga; a0; d;...
            nupi; nuc; nuu; b; sigx2];  
        store_del(isave,:) = del' ;
    end    
end
 
disp( ['MCMC takes '  num2str( etime( clock, start_time) ) ' seconds' ] );
disp(' ' );

pistarhat = mean(store_pistar)';
ustarhat = mean(store_ustar)';
ystarhat = mean(store_ystar)';
hpihat = mean(exp(store_hpi/2))';
hchat = mean(exp(store_hc/2))';
huhat = mean(exp(store_hu/2))';
thetahat = mean(store_theta)';
muhat = mean(store_mu)';
delhat = mean(store_del)';
ahat = mean(store_a)';
accept = [countrhoc countrhou countsigustar2 countsigpistar2 ...
    countsighpi2 countsighc2 countsighu2]/(nsims+burnin);
pistarCI = quantile(store_pistar,[.1 .9])';
ustarCI = quantile(store_ustar,[.1 .9])';
hpiCI = quantile(exp(store_hpi/2),[.1 .9])';
hcCI = quantile(exp(store_hc/2),[.1 .9])';
huCI = quantile(exp(store_hu/2),[.1 .9])';
thetaCI = quantile(store_theta,[.025 .975])';

% plot of graphs
figure;
hold on ; box on ;
area([8 12]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([26 30]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([43 46]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([54 57]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([92 96]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([108 113]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([133 135]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([140 144]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([175 177]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([213 216]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([244 250]-4,[6 6],0,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
plot(1:T,ahat,'LineWidth',1,'Color','blue');
hold off
axis([1, T+1, 0, 5]); box off;  title('Trend output growth');
set(gca,'XTick',[13 53 93 133 173 213 253]-4);
set(gca,'XTickLabel',{'1950','1960','1970','1980','1990','2000','2010'});

figure;
hold on ; box on ;
area([8 12]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([26 30]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([43 46]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([54 57]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([92 96]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([108 113]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([133 135]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([140 144]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([175 177]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([213 216]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
area([244 250]-4,[6 6],-11,'FaceColor',[.91 .91 .91],'EdgeColor','none'); set(gcf,'renderer','painters');
plot(1:T,y-ystarhat, 'LineWidth',1,'Color','blue');
plot(1:T, zeros(T,1),'-k','LineWidth',1);
hold off
axis([1, T+1, -11, 6]); box off; title('Output gap');
set(gca,'XTick',[13 53 93 133 173 213 253]-4);
set(gca,'XTickLabel',{'1950','1960','1970','1980','1990','2000','2010'});

figure;
hold on
plotCI(tid,pistarCI(:,1),pistarCI(:,2));
plot(tid,pistarhat);
hold off
box off; xlim([tid(1)-1 tid(end)+1]);  title('Trend inflation');

figure;
hold on
plotCI(tid,ustarCI(:,1),ustarCI(:,2));
plot(tid,ustarhat);
hold off;
box off; xlim([tid(1)-1 tid(end)+1]); title('NAIRU');    

figure;
subplot(3,1,1);
hold on
plotCI(tid,hpiCI(:,1),hpiCI(:,2));
plot(tid,hpihat);
hold off;
box off; xlim([tid(1)-1 tid(end)+1]); title('Std. dev. of the inflation equation');
subplot(3,1,2);
hold on
plotCI(tid,hcCI(:,1),hcCI(:,2));
plot(tid,hchat);
hold off;
box off; xlim([tid(1)-1 tid(end)+1]); title('Std. dev. of the output equation');
subplot(3,1,3);
hold on
plotCI(tid,huCI(:,1),huCI(:,2));
plot(tid,huhat);
hold off;
box off; xlim([tid(1)-1 tid(end)+1]); title('Std. dev. of the unemployment equation');

    % produce model probabilities - stored in "model_count"
countmodels;
